% Script for generating the PO curve for a given process gp
% The controller used is an a PI controller or a Smith Predictor controller

%clc
clear all

global gn gp msEq iaeWeights manWeights iaeTuning %q tuning

curDir = pwd;
mainDir = fileparts(curDir);
sharedDir = fullfile(mainDir,'sharedFiles');
addpath(sharedDir);

tic


modelId = 2;  % use 1,2 oe 3   
msSpace = 1.25:0.5:3; % the Ms search range        
                
[gp,gn ] = modelHelene(modelId); % getting the model


%% Finding the iaeWeights 

    fprintf('Finding Optimal IAE Weights \n')
    fprintf('Case: %g\n',modelId)
    fprintf('***************************\n')
    fprintf('\n')
    fprintf('costFun(iae) \t minTuning \t\t Ms \t exitFlag \n')
    fprintf('-------------------------------------------------\n')

msEq = 1.59; % Ms for the iaeWeights 
opt = optimset('algorithm','interior-point','Display','off','TolCon',1e-4); %'active-set', 'trust-region-reflective', 'interior-point', 'interior-point-convex', 'levenberg-marquardt', 'trust-region-dogleg', 'lm-line-search', or 'sqp'.
% 

%       1 1 1];   %case 17
  
  
  X0 = [0.7 0.8     %case1
      0.05 1.02       %case2
      1.0 0.7       %case3 
          ];   
  

x0 = X0(modelId,:)';

%% Finding the weights for different transfer functions G 

iaeWeights = [1; 1]; % cost function weigths
manWeights = [1 0];

[minTuningSp,iaeSp,exitFlagSp] = fmincon(@costFun,x0,[],[],[],[],[0;0],[],@conFun,opt);

    fprintf('%0.2f \t \t %.2f  %.2f \t %.2f\t %i \t \n',iaeSp, minTuningSp, msEq, exitFlagSp)
    

% X0 = [0.9 0.5 %case1
%       1.0 0.6  %case2
%       0.8 0.7  %case3
%       2.2 1.5  %case4
%           ];   %case 17
%       
x0 = X0(modelId,:)';
    
manWeights = [0 1];

[minTuningD,iaeD,exitFlagD] = fmincon(@costFun,x0,[],[],[],[],[0;0],[],@conFun,opt);

    fprintf('%0.2f \t \t %.2f  %.2f \t %.2f\t %i \t \n',iaeD, minTuningD, msEq, exitFlagD)
    fprintf('\n');
    fprintf('\n');
    fprintf('\n');
    fprintf('\n');


iaeWeights = [iaeSp; iaeD];


%Weights Optimal Smith Predictor: G=exp(-s)/(s+1)
%  iaeWeights=[    1.6883 ;   1.6754];              %Case 1: G=exp(-s)/(s+1) 
%  iaeWeights=[    1.6961 ;   1.2680];                %Case 3: G=exp(-s)/(8s+1)
%  iaeWeights=[    ? ;   ?];              %Case 2: G=exp(-s)
    

%% Generating curve

i = 1; % iteration counter for command window printout
minTuning = []; % matrices for storing results
costTuning = [];
iaeOptTun = [];

x0 = X0(modelId,:)';

manWeights = [.5, .5];

    fprintf('Generating the PO Curve\n')
    fprintf('***********************\n')
    fprintf('\n')
    fprintf('Number of iterations: %i \n', length(msSpace))
    fprintf('\n')
    fprintf('costFun(%.1f, %.1f) \t minTuning \t \t Ms \t exitFlag \t iterations left \n',manWeights)
    fprintf('--------------------------------------------------------------------------------\n')
    
opt = optimset('algorithm','interior-point','Display','off','TolCon',1e-4); %'active-set', 'trust-region-reflective', 'interior-point', 'interior-point-convex', 'levenberg-marquardt', 'trust-region-dogleg', 'lm-line-search', or 'sqp'.

for msEq = msSpace;

% Optimizing

[minTuningTemp, iaeTuningTemp, exitFlagTuningTemp] = fmincon(@costFun,x0,[],[],[],[],[0;0],[],@conFun,opt);

minTuning(:,i) = minTuningTemp; % storing resutlts
costTuning(i) = iaeTuningTemp;
exitFlagTuning(i) = exitFlagTuningTemp;
iaeOptTun(i,:) = iaeTuning;

%warm-start -> initial guess for optimization
 x0 = minTuningTemp;



fprintf('%0.2f \t \t \t %.2f  %.2f \t %.2f\t %i \t \t\t %i \n',iaeTuningTemp, minTuningTemp, msEq, exitFlagTuningTemp, length(msSpace)-i)

i = i + 1; %updating iteration counter
end

    fprintf('\n')
    fprintf('Calculation Finished!\n')
    fprintf('=====================\n')
toc

%% Plotting

% J versus Ms
figure(modelId)
clf
h(1) = plot(msSpace,costTuning);
axis([1.2 3 0.8 1.3])
xlabel('Robustness, $M_s$', 'interpreter','latex','FontSize',14)
ylabel('Performance, $J$', 'interpreter','latex','FontSize',14)
% titleName = {'$\frac{\tau_1}{\tau_2} = 0.5$'
%              '$\frac{\tau_1}{\tau_2} = 0.8$'
%              '$\frac{\tau_1}{\tau_2} = 0.2$'
%              '$\frac{\tau_1}{\tau_2} = 0.5$, $\tau_2 = 1.5\cdot\theta$'
%              '$\frac{\tau_1}{\tau_2} = 0.8$, $\tau_2 = 1.5\cdot\theta$'
%              '$\frac{\tau_1}{\tau_2} = 0.3$, $\tau_2 = 1.5\cdot\theta$'
%              '$\frac{\tau_1}{\tau_2} = 0.5$, $\tau_2 = 2\cdot\theta$'
%              '$\frac{\tau_1}{\tau_2} = 0.8$, $\tau_2 = 2\cdot\theta$'
%              '$\frac{\tau_1}{\tau_2} = 0.3$, $\tau_2 = 2\cdot\theta$'};
titleName = {'Case 1','Case 2','Case 3','Case 4','Case 5',...
             'Case 6','Case 7','Case 8','Case 9','Case 10',...
             'Case 11','Case 12','Case 13','Case 14','Case 15'...
             'Case 16','Case 17'};

title(titleName{modelId},'interpreter','latex','FontSize',14)


%TV = tvTuning(gp,minTuning);
%% Storing results
modelName = {'case1','case2','case3','case4','case5','case6',...
             'case7','case8','case9','case10','case11','case12',...
             'case13','case14','case15','case16','case17'};
         
res.case = num2str(modelName{modelId});

% res.POpid.minSp.tuning = minTuningSp;
% res.POpid.minSp.iae = iaeSp;
% res.POpid.minSp.exitFlag = exitFlagSp;

% res.POpid.minD.tuning = minTuningD;
% res.POpid.minD.iae = iaeD;
% res.POpid.minD.exitFlag = exitFlagD;

res.POpid.minTuning.ms = msSpace;
res.POpid.minTuning.tuning = minTuning;
res.POpid.minTuning.costFun = costTuning;
res.POpid.minTuning.exitFlag = exitFlagTuning;
res.POpid.minTuning.iae = iaeOptTun;
%res.minTuning.tv = TV;


% pause
% save([mainDir,'\dataFiles\','resPOpid_',num2str(modelName{modelId}),'.mat'],'res') %saving "globaly"
% save(['poPIDresults\','resPOpid_',num2str(modelName{modelId}),'.mat'],'res')       %saving "localy"
%          
% saveas(h(1),['figures\','POpidCurve_',num2str(modelName{modelId}),'.fig'])
% 


restoredefaultpath

